
-- Skill Up plugin Interface
do
	local plugin
	local scrollFrame
	local rank


	local function GetSkillUpChance(id, rank)
		if not rank then
			return 0
		end

		local orange, yellow, green, gray = GnomeWorks:GetSkillLevels(id)

		if rank < yellow then
			return 1
		elseif rank >= gray then
			return 0
		else
			return 1 - (rank - yellow + 1) / (gray - yellow + 1)
		end
	end


	local function ColumnControl(cellFrame,button,source)
		local filterMenuFrame = GnomeWorksMenuFrame
		local scrollFrame = cellFrame:GetParent():GetParent()

		if button == "RightButton" then
			if cellFrame.header.filterMenu then
				local x, y = GetCursorPosition()
				local uiScale = UIParent:GetEffectiveScale()

				EasyMenu(cellFrame.header.filterMenu, filterMenuFrame, UIParent, x/uiScale,y/uiScale, "MENU", 5)
			end
		else
			scrollFrame.sortInvert = (scrollFrame.SortCompare == cellFrame.header.sortCompare) and not scrollFrame.sortInvert
			scrollFrame:HighlightColumn(cellFrame.header.name, scrollFrame.sortInvert)
			scrollFrame.SortCompare = cellFrame.header.sortCompare
			scrollFrame:Refresh()
		end
	end


	local function ColumnTooltip(cellFrame, text)
		GameTooltip:SetOwner(cellFrame, "ANCHOR_TOPLEFT")
		GameTooltip:AddLine(text, 1, 1, 1, true)
		GameTooltip:AddLine("Left-click to Sort")
		GameTooltip:Show()
	end


	local skillUpColumnHeader = {
		name = "Skill Up",
		width = 50,
		headerAlign = "CENTER",
		align = "RIGHT",
		font = "GameFontHighlightSmall",

		sortCompare = function(a, b)
			local s1, s2 = tonumber(a.skillUp or 0), tonumber(b.skillUp or 0)

			if not s1 then
				s1 = abs(tonumber(strsub(a.skillUp, -3)))
			end

			if not s2 then
				s2 = abs(tonumber(strsub(b.skillUp, -3)))
			end

			return s1 - s2
		end,

		draw = function (rowFrame, cellFrame, entry)
			if not entry.subGroup then
				local s = tonumber(entry.skillUp or 0)
				if s then
					cellFrame.text:SetText(format("%.1f%%", s))
					if s == 0 then
						cellFrame.text:SetTextColor(0.50, 0.50, 0.50)
					elseif s < 33 then
						s = s * 3 / 100
						cellFrame.text:SetTextColor(0.50 - 0.25 * s, 0.50 + 0.25 * s, 0.50 - 0.25 * s)
					elseif s < 66 then
						s = (s - 33) * 3 / 100
						cellFrame.text:SetTextColor(0.25 + 0.75 * s, 0.75 + 0.25 * s, 0.25 - 0.25 * s)
					elseif s < 100 then
						s = (s - 66) * 3 / 100
						cellFrame.text:SetTextColor(1.00, 1.00 - 0.50 * s, 0.00 + 0.25 * s)
					else
						cellFrame.text:SetTextColor(1.00, 0.50, 0.25)
					end
				else
					cellFrame.text:SetText(entry.skillUp.."%")
				end
			else
				cellFrame.text:SetTextColor(1, 0.82, 0)
				cellFrame.text:SetText("")
			end
		end,

		OnClick = function (cellFrame, button, source)
			if cellFrame:GetParent().rowIndex <=0 then
				ColumnControl(cellFrame, button, source)
			end
		end,

		OnEnter = function (cellFrame)
			if cellFrame:GetParent().rowIndex <=0 then
				ColumnTooltip(cellFrame, "Skill Up Chance")
			end
		end,

		OnLeave = function (cellFrame)
			if cellFrame:GetParent().rowIndex <=0 then
				GameTooltip:Hide()
			end
		end,

		enabled = function()
			if not plugin.enabled then
				return
			end

			if scrollFrame:IsVisible() then
				local realRank, maxRank, estimatedRank, bonus = GnomeWorks:GetTradeSkillRank(GnomeWorks.player, GnomeWorks.tradeID)
				rank = (estimatedRank or realRank) - (bonus or 0)

				if rank > 0 and realRank < maxRank then
					if not GnomeWorks.data.pseudoTradeData[GnomeWorks.tradeID] then
						return true
					end
				end
			end
		end,
	}


	local function UpdateData(scrollFrame, entry)
		if GnomeWorks.tradeID and GnomeWorks.player then
			local skillName, skillType = GetTradeSkillInfo(entry.index)

			if skillType ~= "header" and entry.recipeID then
				if GnomeWorksDB.recipeSkillLevels then
					entry.skillUp = math.floor(GetSkillUpChance(entry.recipeID, rank) * 10000) / 100
					if entry.skillUp == 100 then
						entry.skillUp = entry.skillUp * (GnomeWorksDB.skillUps[entry.recipeID] or 1)
					end
				else
					if skillType == "optimal" then
						entry.skillUp = "|cffff8040"..((GnomeWorksDB.skillUps[entry.recipeID] or 1) * 100)
					elseif skillType == "medium" then
						entry.skillUp = "|cffffff0050-99"
					elseif skillType == "easy" then
						entry.skillUp = "|cff40c0001-49"
					else
						entry.skillUp = 0
					end
				end
			end
		end
	end


	local function togglePlugin()
		plugin.enabled = not plugin.enabled

		scrollFrame:Refresh()
	end


	local function Init()
		scrollFrame = GnomeWorks:GetSkillListScrollFrame()
		scrollFrame:RegisterRowUpdate(UpdateData, plugin)

		local skillUpColumn = scrollFrame:AddColumn(skillUpColumnHeader, plugin)
		local button = plugin:AddButton("Enabled", togglePlugin)
		button.checked = function() return plugin.enabled end

		return true
	end


	plugin = GnomeWorks:RegisterPlugin("SkillUp Chance", Init)
end
